/*

    Copyright (c) 2018 - 2020 Analog Flavor EURL

    Contact us for support:
        http://www.analogflavor.com

-----------------------------------------------------------------------------
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

    * Neither the name of Analog Flavor EURL nor 
      the names of its contributors may be used to endorse or promote products derived from
      this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
-----------------------------------------------------------------------------

*/


#include <wiener_mpod_lv_hv_snmp.h>

#include <string.h>
#include <math.h>
#include <stdio.h>
#ifdef WIN32
#include <Windows.h>
void sleep_ms(int ms) {
    Sleep(ms);
}
#else
#include <unistd.h>
void sleep_ms(int ms) {
    usleep(ms * 1000);
}
#endif

int checkBitIsSet(int status_flags, int bit) {
    return status_flags & (1 << bit);
}

void hvChannelTest(void *obj_ptr, int module_slot_index, int channel_index, int thread_polling_interval_ms) {
    double double_value;
    int int_value;
    int wait_delay;
    double meas_v, meas_i;
    int channel_status;

    /* we must wait until we get feedback for the executed commands */
    wait_delay = 2 * thread_polling_interval_ms;
    if (wait_delay < 500) {
        wait_delay = 500;
    }
    printf("testing HV channel %d in module %d\n", channel_index, module_slot_index);

    /* setting some values */
    double_value = 100.0;
    int_value = mpod_snmp_channelSetVoltage(obj_ptr, module_slot_index, channel_index, double_value);

    printf("  switching channel on\n");
    int_value = 1;
    int_value = mpod_snmp_channelSetSwitch(obj_ptr, module_slot_index, channel_index, int_value);
    
    /* wait until channel has ramped up */
    sleep_ms(wait_delay);
    channel_status = 2048;
    /* testing status flag "ramping up" */
    while (checkBitIsSet(channel_status, 11)) {
        sleep_ms(wait_delay);
        /* reading channel status */
        channel_status = mpod_snmp_channelGetStatus(obj_ptr, module_slot_index, channel_index);
        /* reading measured values */
        meas_v = mpod_snmp_channelGetMeasurementSenseVoltage(obj_ptr, module_slot_index, channel_index);
        meas_i = mpod_snmp_channelGetMeasurementCurrent(obj_ptr, module_slot_index, channel_index);
        printf("  raming up (%g V, %g A)\n", meas_v, meas_i);
    }
    /* switch off */
    printf("  switching channel off\n");
    int_value = 0;
    int_value = mpod_snmp_channelSetSwitch(obj_ptr, module_slot_index, channel_index, int_value);

    /* wait until channel has ramped down */
    sleep_ms(wait_delay);
    channel_status = 4096;
    /* testing status flag "ramping down" */
    while (checkBitIsSet(channel_status, 12)) {
        sleep_ms(wait_delay);
        /* reading channel status */
        channel_status = mpod_snmp_channelGetStatus(obj_ptr, module_slot_index, channel_index);
        /* reading measured values */
        meas_v = mpod_snmp_channelGetMeasurementSenseVoltage(obj_ptr, module_slot_index, channel_index);
        meas_i = mpod_snmp_channelGetMeasurementCurrent(obj_ptr, module_slot_index, channel_index);
        printf("  raming down (%g V, %g A)\n", meas_v, meas_i);
    }
}

int main(int argc, char **argv) {

    void *obj_ptr;
    const char *string_value;
    int int_value;
    const int *occupied_slots;
    const int *occupied_slots_iterator;
    unsigned module_slot_index, nb_channels_per_module, channel_index;
    int thread_polling_interval_ms = 1000; /* set to 0 for debug */

    if (!mpod_snmp_winsock_startup()) {
        printf("Winsock startup failed.\n");
        getchar();
        return 1;
    }

    /* create object */
    obj_ptr = mpod_snmp_create();

    /*  establish connection */
    if (argc >= 2) {
        /* the port ans community can also specified as  "192.168.0.101:161:guru" only in the first argument. */
        mpod_snmp_crateConnect(obj_ptr, argv[1], 0, "", 0.5 * thread_polling_interval_ms, thread_polling_interval_ms, 2.0 * thread_polling_interval_ms);
    }
    else {
        mpod_snmp_crateConnect(obj_ptr, "192.168.0.101", 10161, "guru", 0.5 * thread_polling_interval_ms, thread_polling_interval_ms, 2.0 * thread_polling_interval_ms);
    }

    /* check that the connection has been established */
    if (mpod_snmp_crateCheckConnectionIsOk(obj_ptr)) {

        /* make sure all data has been read to the buffer */
        sleep_ms(3 * thread_polling_interval_ms);

        /* get the list of slots occupied by a LV or HV module */
        /* the array is terminated by a -1 */
        occupied_slots = mpod_snmp_crateGetOccupiedSlots(obj_ptr);
        if (occupied_slots) {
            occupied_slots_iterator = occupied_slots;

            /* traverse the list of occupied slots */
            while (*occupied_slots_iterator >= 0) {
                module_slot_index = *occupied_slots_iterator;
                /* find out if the module is a HV module or not */
                if (mpod_snmp_moduleGetIsHvModule(obj_ptr, module_slot_index)) {
                    /* traverse all channels */
                    nb_channels_per_module = mpod_snmp_moduleGetNumberOfChannels(obj_ptr, module_slot_index);
                    for (channel_index = 0; channel_index < nb_channels_per_module; channel_index++) {
                        /* perform test on channels */
                        hvChannelTest(obj_ptr, module_slot_index, channel_index, thread_polling_interval_ms);
                    }
                }

                occupied_slots_iterator++;
            }
        }
        else {
            printf("CrateGetOccupiedSlots() fails.\n");
        }

    }
    else {

        /* print error messages */
        printf("CrateCheckConnectionIsOk() returns false.\n");
        int_value = mpod_snmp_crateGetConnectionStatus(obj_ptr);
        printf("CrateGetConnectionStatus() returns %d.\n", int_value);
        string_value = mpod_snmp_crateGetConnectionStatusAsString(obj_ptr);
        printf("CrateGetConnectionStatusAsString() returns %s.\n", string_value);
        string_value = mpod_snmp_crateGetErrorString(obj_ptr);
        printf("CrateGetErrorString() returns %s.\n", string_value);

    }

    /* delete object */
    mpod_snmp_delete(obj_ptr);

    mpod_snmp_winsock_cleanup();

#ifdef WIN32
    getchar();
#endif

    return 0;
}


